/*==============================================================================


       eg_6 - CList Manipulations Examples

       this code illustrates a variety of listbox classes, ready to incorporate
       or modify in your own programs:

       -string listbox, a good general-purpose list display

       -dynamic listbox, where the list data is generated on the fly

       -structured listbox, using a structure for input data & displaying images


==============================================================================*/
#include "gui.h"
#include "ebm_object.h"

extern "C"
{
#include "ereader_hostio.h"
}
#include "eg_6_form.h"
/*==============================================================================

       The Choice variable that each listbox class sets on exit is declared.
       Good programming would dictate making it a member of each class, but the
       purpose of this code is to teach using the listbox, so this extra coding
       has been left out

==============================================================================*/
static int Choice=0;
/*==============================================================================

      CStringList/CListPopUp - These classes form a simple popup box ideal for
          general use - it lets you set the list to an array of string pointers,
          and returns the index selected (or -1 if cancelled)

==============================================================================*/
// here is our test data, showing how it is set up
extern const char *g_strings[]= { "Line A", "Line B", "Line C", "Line D",
    "Line E", "Line F", "Line G", "Line H", "Line I", "Line J", "Line K" };
//------------------------------------------------------------------------------
// now calc size of array - size of whole array / size of individual element
const int g_totalStrings = sizeof(g_strings) / sizeof(g_strings[0]);
//------------------------------------------------------------------------------
//     CStringList
//------------------------------------------------------------------------------
class CStringList : public CList
{

  // basic override of CList for display of strings, one per line

  private:

    const char **m_entry; // pointer to array of strings

    CWindow *m_window;
    int m_rowHeight;   // value cached for use in GetRowHeight() for speed

    U16 GetRowHeight(S32 row);
    void DrawRow(RECT *rect, S32 row);

  public:

    CStringList(int id,int dx,int dy,const char **pointerArray,int numRows);

};
//------------------------------------------------------------------------------
CStringList::CStringList(int id,int dx,int dy,const char **pointerArray,int numRows)
 : CList(id,dx,dy,numRows,0),
   m_entry(pointerArray),
   m_window(0),
   m_rowHeight(0)
{
}
//------------------------------------------------------------------------------
U16 CStringList::GetRowHeight(S32 row)
{
  // rather than calc the row height each time, for speed we save the value the
  // first time in here, as well as the window pointer
  if ( NULL==m_window ) // first time in?
  {
    m_window=GetWindow();
    m_rowHeight=GUI_FontHeight(m_window->GetFont());
  }
  return m_rowHeight;
}
//------------------------------------------------------------------------------
void CStringList::DrawRow(RECT *rect, S32 row)
{
  // given the rectangle describing the row, draw it
  m_window->DrawText(m_entry[row], rect->x, rect->y);
}
//------------------------------------------------------------------------------
//     CListPopUp - wrapper for CStringList
//------------------------------------------------------------------------------
class CListPopUp : public CWindow
{

  private:

    // message handler - we use it to detect whether row selected or cancelled
    S32 MsgHandler( MSG_TYPE type, CViewable *from, S32 data );

  public:

    CListPopUp(int x,int y,int dx,int dy,
               const char**array,int total,
               const char *caption);

};
//------------------------------------------------------------------------------
CListPopUp::CListPopUp(int x,int y,int dx,int dy,
                       const char**array,int total,
                       const char *caption)
 : CWindow(100,x,y,dx,dy,caption,false,false,false)
{
  // window is one big listbox, with caption at top, and cancel button at bottom
  AddChild( new CButton(101,dx-8,20,"Cancel"),            2, dy-40);
  AddChild( new CStringList(102,dx-10,dy-48,array,total), 5,     5);
}
//------------------------------------------------------------------------------
S32 CListPopUp::MsgHandler( MSG_TYPE type, CViewable *from, S32 data )
{
  if ( MSG_ROW_SELECT==type ) // listbox choice made?
  {
    Choice=data; // then save it
    Close();
  }
  else if ( MSG_BUTTON_SELECT==type ) // cancel button?
  {
    Choice=-1; // then set flag to an invalid index
    Close();
  }
  return CWindow::MsgHandler(type, from, data);
}
/*==============================================================================

       CDynamicList/CDynamicPopUp - class that shows you don't need data to
           create a listbox - the program generates a range of values from
           the two numbers passed in, and can be handy for listing a range
           of values

==============================================================================*/
class CDynamicList : public CList
{


  private:

    CWindow *m_window;
    int m_rowHeight;
    int m_firstValue; // start of numbers to display

    U16 GetRowHeight(S32 row);
    void DrawRow(RECT *rect, S32 row);

  public:

    CDynamicList(int id,int dx,int dy,int firstValue,int numRows);

};
//------------------------------------------------------------------------------
CDynamicList::CDynamicList(int id,int dx,int dy,int firstValue,int numRows)
 : CList(id,dx,dy,numRows,0),
   m_window(0),
   m_rowHeight(0),
   m_firstValue(firstValue)
{
}
//------------------------------------------------------------------------------
U16 CDynamicList::GetRowHeight(S32 row)
{
  if ( NULL==m_window )
  {
    m_window=GetWindow();
    m_rowHeight=GUI_FontHeight(m_window->GetFont());
  }
  return m_rowHeight;
}
//------------------------------------------------------------------------------
void CDynamicList::DrawRow(RECT *rect, S32 row)
{
  char buff[99]="";
  sprintf(buff,"Item #%ld",row+m_firstValue);
  m_window->DrawText(buff, rect->x, rect->y);
}
//------------------------------------------------------------------------------
//     CDynamicPopUp - wrapper for CDynamicList
//------------------------------------------------------------------------------
class CDynamicPopUp : public CWindow
{

  private:

    S32 MsgHandler( MSG_TYPE type, CViewable *from, S32 data );

  public:

    CDynamicPopUp(int x,int y,int dx,int dy,int firstItem,int lastItem,const char *caption);

};
//------------------------------------------------------------------------------
CDynamicPopUp::CDynamicPopUp(int x,int y,int dx,int dy,
                           int firstItem,int totalItems,
                           const char *caption)
 : CWindow(100,x,y,dx,dy,caption,false,false,false)
{
  AddChild( new CButton(101,dx-8,20,"Cancel"),                      2, dy-40);
  AddChild( new CDynamicList(102,dx-10,dy-48,firstItem,totalItems), 5,     5);
}
//------------------------------------------------------------------------------
S32 CDynamicPopUp::MsgHandler( MSG_TYPE type, CViewable *from, S32 data )
{
  if ( MSG_ROW_SELECT==type )
  {
    Choice=data;
    Close();
  }
  else if ( MSG_BUTTON_SELECT==type )
  {
    Choice=-1;
    Close();
  }
  return CWindow::MsgHandler(type, from, data);
}
/*==============================================================================


      CStructuredList/CStructuredPopUp - This listbox is customized with a
          structure, which we've created statically. It not only shows that
          the listbox data doesn't have to be a string, but also shows how
          images can be combined with text in the DrawRow() function. As a
          side benefit, it also shows how you can set up data in your
          program that is easy to maintain; by grouping all relevant info
          in one spot, code maintanence is easier

==============================================================================*/
#include "images.cpp"
//------------------------------------------------------------------------------
// data passed for list rows  - description, picture, and angle represented
struct s_structData
{
  const char *caption;
  const IMAGE *image;
  int degree;
} extern const g_entries[]= {
  { "North",      &img_N,    0 },
  { "North-East", &img_NE,  45 },
  { "East",       &img_E,   90 },
  { "South-East", &img_SE, 135 },
  { "South",      &img_S,  180 },
  { "South-West", &img_SW, 225 },
  { "West",       &img_W,  270 },
  { "North-West", &img_NW, 315 } };
const int g_totalEntries = sizeof(g_entries) / sizeof(g_entries[0]);
//------------------------------------------------------------------------------
//     CStructuredList
//------------------------------------------------------------------------------
class CStructuredList : public CList
{


  private:

    CWindow *m_window;
    int m_rowHeight;
    const s_structData *m_array; // pointer to our structure's data

    U16 GetRowHeight(S32 row);
    void DrawRow(RECT *rect, S32 row);

  public:

    CStructuredList(int id,int dx,int dy,const s_structData *ptr,int numRows);

};
//------------------------------------------------------------------------------
CStructuredList::CStructuredList(int id,int dx,int dy,
                                 const s_structData *ptr,int numRows)
 : CList(id,dx,dy,numRows,0),
   m_window(0),
   m_array(ptr)
{
  // unlike the other listboxes, we can precalculate the row height, since
  // the images are all larger than the font we will use, and they are all
  // the same height
  m_rowHeight=ptr[0].image->img_height+2; // add a little spacing
}
//------------------------------------------------------------------------------
U16 CStructuredList::GetRowHeight(S32 row)
{
  return m_rowHeight;
}
//------------------------------------------------------------------------------
void CStructuredList::DrawRow(RECT *rect, S32 row)
{
  // we want to draw an image, followed by the text - note that the window
  // pointer wasn't set in GetRowHeight() like we did in the other listboxes,
  // so we test and set it here
  if ( NULL==m_window ) // first time in?
    m_window=GetWindow(); // store value to avoid repeatedly calling GetWindow()
  m_window->DrawImage( rect->x,
                       rect->y,
                       m_array[row].image,
                       0,0,0,0,
                       m_array[row].image->img_width,
                       m_array[row].image->img_height,
                       IMG_CMB_SRC);
  m_window->DrawText( m_array[row].caption, // display text just past image
                      rect->x+m_array[row].image->img_width+3,
                      rect->y+3);
}
//------------------------------------------------------------------------------
//     CStructuredPopUp - wrapper for CStructuredList
//------------------------------------------------------------------------------
class CStructuredPopUp : public CWindow
{

  private:

    S32 MsgHandler( MSG_TYPE type, CViewable *from, S32 data );

  public:

    CStructuredPopUp(int x,int y,int dx,int dy,
                     const s_structData *ptr,int numRows,
                     const char *caption);

};
//------------------------------------------------------------------------------
CStructuredPopUp::CStructuredPopUp(int x,int y,int dx,int dy,
                                   const s_structData *ptr,int numRows,
                                   const char *caption)
 : CWindow(100,x,y,dx,dy,caption,false,false,false)
{
  AddChild( new CButton(101,dx-8,20,"Cancel"),                2,dy-40);
  AddChild( new CStructuredList(102,dx-10,dy-48,ptr,numRows), 5,    5);
}
//------------------------------------------------------------------------------
S32 CStructuredPopUp::MsgHandler( MSG_TYPE type, CViewable *from, S32 data )
{
  if ( MSG_ROW_SELECT==type )
  {
    Choice=data;
    Close();
  }
  else if ( MSG_BUTTON_SELECT==type )
  {
    Choice=-1;
    Close();
  }
  return CWindow::MsgHandler(type, from, data);
}
/*==============================================================================

       main window - all we have to do is wait for button presses

==============================================================================*/
S32 CWindow1::MsgHandler( MSG_TYPE type, CViewable *from, S32 data )
{
  if ( MSG_BUTTON_SELECT==type ) // our button press?
  {
    switch (from->GetID() )
    {
      //------------------------------------------------------------------------
      //  string list popup
      //------------------------------------------------------------------------
      case CWindow1_Button1:
      {
        // call our listbox, and don't return until we've decided - often
        // referred to as a modal dialog box
        GUI_EventLoop(new CListPopUp(15,20,170,200,
                                     g_strings,g_totalStrings,
                                     "String List Popup"));
        // now display our choice
        char message[99]="";
        if (Choice<0) // variable declared at start of source, and set by listbox
          strcpy(message,"You didn't chose anything!");
        else
          sprintf(message,"You selected row %d which is '%s'",
                          Choice,g_strings[Choice]);
        GUI_Alert(ALERT_WARNING,message);
      }
        break;
      //------------------------------------------------------------------------
      //  dynamic list popup
      //------------------------------------------------------------------------
      case CWindow1_Button2: // psuedo list
      {
        // very easy listbox - just pass it a range
        int startNumber=100, totalNumbers=50;
        GUI_EventLoop(new CDynamicPopUp(15,20,170,200,
                                       startNumber,totalNumbers,
                                       "Psuedo List (Dynamic Data)"));
        // now display our choice
        char message[99]="";
        if (Choice<0)
          strcpy(message,"You didn't chose anything!");
        else
          sprintf(message,"You selected row %d which is %d",
                          Choice,
                          startNumber+Choice);
        GUI_Alert(ALERT_WARNING,message);
      }
        break;
      //------------------------------------------------------------------------
      //  structured list popup
      //------------------------------------------------------------------------
      case CWindow1_Button3:
      {
        GUI_EventLoop(new CStructuredPopUp(15,20,170,200,
                                           g_entries,
                                           g_totalEntries,
                                           "Structure List & Images"));
        // now display our choice
        char message[99]="";
        if (Choice<0)
          strcpy(message,"You didn't chose anything!");
        else
          sprintf(message,"You selected row %d  which is '%s', or %d degrees",
                  Choice,
                  g_entries[Choice].caption,
                  g_entries[Choice].degree );
        GUI_Alert(ALERT_WARNING,message);
      }
        break;
    }
  }
  return CWindow::MsgHandler(type, from, data);
}
/*==============================================================================


==============================================================================*/
static CWindow1 *f_mainWindow;
//------------------------------------------------------------------------------
S32 GUI_main( MSG_TYPE type, CViewable *object, S32 data )
{
  switch (type)
  {
    case MSG_APP_START:
      if (OS_is_present && hostIO_am_I_the_current_task())
        HOSTIO_INLINE_BREAKPOINT();
      f_mainWindow = new CWindow1();
      return 1;
    case MSG_APP_STOP: // prog exit & cleanup
      delete f_mainWindow;
      return 1;
    default:
      break;
  }
  return 0;
};
//------------------------------------------------------------------------------


